interface MetaData {
    start: {
        line: number;
        column: number;
        offset: number;
    };
    end: {
        line: number;
        column: number;
        offset: number;
    };
}

interface IRegExpFlags {
    value: string;
    literal: string;
}

/**
 * RawToken is converted into IToken by the parser.ts convertToken() function.
 * Hence, this could be called a ParserToken.
 */
interface IToken {
    type?: number | 'BlockComment' | 'LineComment';
    value?: number | string;
    lineNumber?: number;
    lineStart?: number;
    literal?: IRegExpFlags;
    octal?: boolean;
    regex?: {
        pattern: string;
        flags: string;
    };
    start?: number;
    startLineNumber?: number;
    startLineStart?: number;
    end?: number;
    loc?: SourceLocation;
    range?: [number, number];
}

interface Position {
    line: number;
    column: number;
}
interface SourceLocation {
    start: Position;
    end: Partial<Position>;
    source?: string;
}

interface GenerateOptions {
    parse?: any;
    comment?: boolean;
    file?: string;
    format?: {
        indent?: {
            style: string;
            base: number;
            adjustMultilineComment: boolean;
        };
        newline?: '\n';
        space?: ' ';
        json?: boolean;
        renumber?: boolean;
        hexadecimal?: boolean;
        quotes?: 'single' | 'double';
        escapeless?: boolean;
        compact?: boolean;
        parentheses?: boolean;
        semicolons?: boolean;
        safeConcatenation?: boolean;
        preserveBlankLines?: boolean;
    };
    moz?: {
        comprehensionExpressionStartsWithAssignment: false;
        starlessGenerator: false;
    };
    sourceContent?: string;
    sourceMap?: null;
    sourceMapRoot?: null;
    sourceMapWithCode?: false;
    directive?: false;
    raw?: true;
    verbatim?: null;
    sourceCode?: null;
}

interface ParseOptions {
    comment?: boolean;
    attachComment?: boolean;
    sourceType?: 'module' | 'script';
    jsx?: boolean;
    /**
     * Determines whether the parser reports range: [start: number, end: number].
     */
    range?: boolean;
    /**
     * Determines whether the parser reports loc: {start:{line:number;column:number}; end:{line:number;column:number}}
     */
    loc?: boolean;
    tokens?: boolean;
    tolerant?: boolean;
    source?: string;
}
interface ParseDelegate {
    (node: IToken, metadata?: MetaData): IToken;
}

/**
 * Provides the MathScript module
 */
/**
 * Determines the transformation action for operators.
 */
interface OperatorTransform {
    /**
     * The nature of the transformation.
     */
    kind: 'rename' | 'function';
    /**
     * The transformed name.
     */
    name: string;
}
/**
 *
 */
interface TranspileOptions extends ParseOptions {
    timeout?: number;
    noLoopCheck?: boolean;
    operatorOverloading?: boolean;
    namespace: string;
    /**
     * The replacements of unary operators that will be made.
     */
    binOp: {
        [name: string]: OperatorTransform;
    };
    /**
    * The replacements of unary operators that will be made.
    */
    unaryOp: {
        [name: string]: OperatorTransform;
    };
}
/**
 * This is the function that we export.
 */
declare function jsToMathScript(code: string, transpileOptions: TranspileOptions, delegate?: ParseDelegate, generateOptions?: GenerateOptions): {
    code: string;
};

export { GenerateOptions, OperatorTransform, TranspileOptions, jsToMathScript };
